@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,13 +15,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Evaluation of PAC scripts."""
 
 import sys
 import functools
-import typing
+from typing import Optional
 
 from PyQt5.QtCore import QObject, pyqtSignal, pyqtSlot, QUrl
 from PyQt5.QtNetwork import (QNetworkProxy, QNetworkRequest, QHostInfo,
@@ -29,7 +29,7 @@ from PyQt5.QtNetwork import (QNetworkProxy, QNetworkRequest, QHostInfo,
                              QHostAddress)
 from PyQt5.QtQml import QJSEngine, QJSValue
 
-from qutebrowser.utils import log, utils, qtutils
+from qutebrowser.utils import log, utils, qtutils, resources
 
 
 class ParseProxyError(Exception):
@@ -190,7 +190,7 @@ class PACResolver:
         self._engine.globalObject().setProperty(
             "PAC", self._engine.newQObject(self._ctx))
         self._evaluate(_PACContext.JS_DEFINITIONS, "pac_js_definitions")
-        self._evaluate(utils.read_file("javascript/pac_utils.js"), "pac_utils")
+        self._evaluate(resources.read_file("javascript/pac_utils.js"), "pac_utils")
         proxy_config = self._engine.newObject()
         proxy_config.setProperty("bindings", self._engine.newObject())
         self._engine.globalObject().setProperty("ProxyConfig", proxy_config)
@@ -251,8 +251,7 @@ class PACFetcher(QObject):
         url.setScheme(url.scheme()[len(pac_prefix):])
 
         self._pac_url = url
-        self._manager = QNetworkAccessManager(
-        )  # type: typing.Optional[QNetworkAccessManager]
+        self._manager: Optional[QNetworkAccessManager] = QNetworkAccessManager()
         self._manager.setProxy(QNetworkProxy(QNetworkProxy.NoProxy))
         self._pac = None
         self._error_message = None
@@ -302,7 +301,7 @@ class PACFetcher(QObject):
         if self._manager is not None:
             loop = qtutils.EventLoop()
             self.finished.connect(loop.quit)
-            loop.exec_()
+            loop.exec()
 
     def fetch_error(self):
         """Check if PAC script is successfully fetched.
